// MT19937 32bits PRNG implementation
// Stefano Barbato - codesink.org
// run:
//		go run main.go
package main

import (
	"fmt"
)

type MT19937 struct {
	MT                  []uint32
	a, b, c, d, f, l, m uint32
	n, r, s, t, u, w    uint32
	index               uint32
}

func NewMT19937(seed uint32) *MT19937 {

	m := new(MT19937)

	// static coefficiens for the 32 bits version
	m.w, m.n, m.m, m.r = 32, 624, 397, 31
	m.a = 0x9908B0DF
	m.u, m.d = 11, 0xFFFFFFFF
	m.s, m.b = 7, 0x9D2C5680
	m.t, m.c = 15, 0xEFC60000
	m.l = 18
	m.f = 1812433253

	m.MT = make([]uint32, m.n)
	m.index = m.n + 1

	m.seed(seed)

	return m
}

func (m *MT19937) seed(v uint32) {
	m.index = m.n
	m.MT[0] = v
	for i := uint32(1); i < m.n; i++ {
		m.MT[i] = (m.f*(m.MT[i-1]^(m.MT[i-1]>>(m.w-2))) + i)
	}
}

func (m *MT19937) ExtractNumber() uint32 {

	if m.index >= m.n {
		if m.index > m.n {
			panic("you must call seed() first")
		}
		m.twist()
	}

	y := uint32(m.MT[m.index])
	y ^= (y >> m.u) & m.d
	y ^= (y << m.s) & m.b
	y ^= (y << m.t) & m.c
	y ^= y >> m.l

	m.index++

	return y
}

func (m *MT19937) twist() {

	lower_mask := uint32((1 << m.r) - 1)
	upper_mask := uint32(^lower_mask)

	for i := uint32(0); i < m.n; i++ {

		x := (m.MT[i] & upper_mask) + (m.MT[(i+1)%m.n] & lower_mask)
		xA := x >> 1
		if x%2 != 0 {
			xA ^= m.a
		}
		m.MT[i] = m.MT[(i+m.m)%m.n] ^ xA
	}

	m.index = 0
}

func main() {
	m := NewMT19937(5489)
	for i := 0; i < 10; i++ {
		fmt.Println(m.ExtractNumber())
	}
}

